package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.TauvAgency;
import com.taauav.admin.mapper.TauvAgencyMapper;
import com.taauav.admin.query.AgencyQuery;
import com.taauav.admin.service.ITauvAgencyService;
import com.taauav.admin.vo.TauvAgencyListVo;
import com.taauav.common.bean.Response;
import com.taauav.common.domain.Entity;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import com.taauav.common.util.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 用户组表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@Service
public class TauvAgencyServiceImpl extends BaseServiceImpl<TauvAgencyMapper, TauvAgency> implements ITauvAgencyService {

    @Autowired
    private TauvAgencyMapper tauvAgencyMapper;

    @Autowired
    private Response response;

    /**
     * 获取机构列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(AgencyQuery query) {
        // 查询条件
        QueryWrapper<TauvAgency> queryWrapper = new QueryWrapper<>();
        // 机构名称
        if (!StringUtils.isEmpty(query.getName())) {
            queryWrapper.like("name", query.getName());
        }
        queryWrapper.eq("pid", 0);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        // 查询条件
        IPage<TauvAgency> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<TauvAgency> data = tauvAgencyMapper.selectPage(page, queryWrapper);
        List<TauvAgency> agencyList = data.getRecords();
        List<TauvAgencyListVo> tauvAgencyListVoList = new ArrayList<>();
        if (!agencyList.isEmpty()) {
            agencyList.forEach(item -> {
                TauvAgencyListVo tauvAgencyListVo = new TauvAgencyListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, tauvAgencyListVo);
                // 获取子级
                tauvAgencyListVo.setChildren(this.getChildAgencyList(item.getId()));
                tauvAgencyListVoList.add(tauvAgencyListVo);
            });
        }
        return response.success(tauvAgencyListVoList);
    }

    /**
     * 获取子级机构
     *
     * @param pid 上级ID
     * @return
     */
    private List<TauvAgencyListVo> getChildAgencyList(Integer pid) {
        QueryWrapper<TauvAgency> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        List<TauvAgencyListVo> agencyListVoList = new ArrayList<>();
        List<TauvAgency> agencyList = tauvAgencyMapper.selectList(queryWrapper);
        if (!agencyList.isEmpty()) {
            agencyList.forEach(item -> {
                // 菜单列表Vo
                TauvAgencyListVo agencyListVo = new TauvAgencyListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, agencyListVo);
                // 获取子级
                List<TauvAgencyListVo> childAgencyList = this.getChildAgencyList(item.getId());
                if (childAgencyList != null) {
                    agencyListVo.setChildren(childAgencyList);
                }
                agencyListVoList.add(agencyListVo);
            });
        }
        return agencyListVoList;
    }

    /**
     * 添加机构
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(TauvAgency entity) {
        if (entity == null) {
            return response.failure("实体对象不能为空");
        }
        if (entity.getPid() == 0) {
            entity.setLevel(1);
        } else {
            TauvAgency agency = tauvAgencyMapper.selectById(entity.getPid());
            if (entity != null) {
                entity.setLevel(agency.getLevel() + 1);
            }
        }
        boolean result = this.addData(entity);
        if (!result) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 获取机构详情
     *
     * @param id 机构ID
     * @return
     */
    @Override
    public Response info(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("机构ID不存在");
        }
        TauvAgency entity = this.getInfo(id);
        if (entity == null) {
            return response.failure("机构信息不存在");
        }
        return response.success(entity);
    }

    /**
     * 编辑机构
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(TauvAgency entity) {
        if (entity == null || entity.getId() == null) {
            return response.failure("实体对象不能为空");
        }
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 删除机构
     *
     * @param id 机构ID
     * @return
     */
    @Override
    public Response delete(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("机构ID不存在");
        }
        TauvAgency entity = tauvAgencyMapper.selectById(id);
        if (entity == null) {
            return response.failure("机构信息不存在");
        }
        boolean result = this.deleteById(id, entity);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response setStatus(TauvAgency entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("设备ID不存在");
        }
        if (entity.getStatus() == null || entity.getStatus() <= 0) {
            return response.failure("状态不存在");
        }
        int result = tauvAgencyMapper.updateById(entity);
        if (result == 0) {
            return response.failure("设置失败");
        }
        return response.success("设置成功");
    }

    /**
     * 获取机构列表
     *
     * @param pid 上级ID
     * @return
     */
    @Override
    public List<TauvAgencyListVo> getAgencyList(Integer pid) {
        // 查询条件
        QueryWrapper<TauvAgency> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("status", 1);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        // 查询数据
        List<TauvAgency> tauvAgencyList = tauvAgencyMapper.selectList(queryWrapper);
        List<TauvAgencyListVo> tauvAgencyListVoList = new ArrayList<>();
        if (!tauvAgencyList.isEmpty()) {
            tauvAgencyList.forEach(item -> {
                TauvAgencyListVo tauvAgencyListVo = new TauvAgencyListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, tauvAgencyListVo);
                tauvAgencyListVoList.add(tauvAgencyListVo);
                // 获取子级机构
                List<TauvAgencyListVo> children = getAgencyList(item.getId());
                if (!children.isEmpty()) {
                    children.forEach(subItem -> {
                        subItem.setName("|--" + subItem.getName());
                        tauvAgencyListVoList.add(subItem);
                    });
                }
            });
        }
        return tauvAgencyListVoList;
    }
}
