package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.constant.CommonConstant;
import com.taauav.common.bean.Response;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import com.taauav.common.util.StringUtils;
import com.taauav.front.entity.LsAdmin;
import com.taauav.front.mapper.LsAdminMapper;
import com.taauav.front.query.LsAdminQuery;
import com.taauav.front.service.ILsAdminService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.taauav.front.vo.LsAdminListVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 外部人员表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@Service
public class LsAdminServiceImpl extends ServiceImpl<LsAdminMapper, LsAdmin> implements ILsAdminService {

    @Autowired
    private LsAdminMapper lsAdminMapper;

    @Autowired
    private Response response;

    /**
     * 获取数据列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(LsAdminQuery query) {
        // 查询条件
        QueryWrapper<LsAdmin> queryWrapper = new QueryWrapper<>();
        // 用户名称
        if (!StringUtils.isEmpty(query.getRealname())) {
            queryWrapper.like("realname", query.getRealname());
        }
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        // 查询数据
        IPage<LsAdmin> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<LsAdmin> data = lsAdminMapper.selectPage(page, queryWrapper);
        List<LsAdmin> lsAdminList = data.getRecords();
        List<LsAdminListVo> lsAdminListVoList = new ArrayList<>();
        if (!lsAdminList.isEmpty()) {
            lsAdminList.forEach(item -> {
                LsAdminListVo lsAdminListVo = new LsAdminListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, lsAdminListVo);
                lsAdminListVoList.add(lsAdminListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", lsAdminListVoList);
        return response.success(result);
    }

    /**
     * 添加用户
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(LsAdmin entity) {
        entity.setCreateUser(ShiroUtils.getAdminId());
        entity.setCreateTime(FunctionUtils.getCurrentTime());
        if (StringUtils.isEmpty(entity.getPassword())) {
            // 初始密码:123456
            entity.setPassword(FunctionUtils.password("123456"));
        } else {
            entity.setPassword(FunctionUtils.password(entity.getPassword()));
        }
        int result = lsAdminMapper.insert(entity);
        if (result == 0) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 获取用户详情
     *
     * @param id 用户ID
     * @return
     */
    @Override
    public Response info(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("用户ID不存在");
        }
        LsAdmin entity = lsAdminMapper.selectById(id);
        if (entity == null) {
            return response.failure("用户信息不存在");
        }
        return response.success(entity);
    }

    /**
     * 编辑用户
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(LsAdmin entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("用户ID不存在");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(FunctionUtils.getCurrentTime());
        // 设置密码并加密
        if (!StringUtils.isEmpty(entity.getPassword())) {
            entity.setPassword(FunctionUtils.password(entity.getPassword()));
        }
        int result = lsAdminMapper.updateById(entity);
        if (result == 0) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 删除用户
     *
     * @param id 用户ID
     * @return
     */
    @Override
    public Response delete(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("用户ID不存在");
        }
        LsAdmin entity = lsAdminMapper.selectById(id);
        if (entity == null) {
            return response.failure("用户信息不存在");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(FunctionUtils.getCurrentTime());
        entity.setMark(0);
        int result = lsAdminMapper.updateById(entity);
        if (result == 0) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response setStatus(LsAdmin entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("用户ID不存在");
        }
        if (entity.getStatus() == null || entity.getStatus() <= 0) {
            return response.failure("用户状态不能为空");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(FunctionUtils.getCurrentTime());
        int result = lsAdminMapper.updateById(entity);
        if (result == 0) {
            return response.failure("设置失败");
        }
        return response.success("设置成功");
    }

    /**
     * 重置密码
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response resetPwd(LsAdmin entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("用户ID不存在");
        }
        // 重置密码：123456
        entity.setPassword(FunctionUtils.password("123456"));
        int result = lsAdminMapper.updateById(entity);
        if (result == 0) {
            return response.failure("重置失败");
        }
        return response.success("重置成功");
    }

    /**
     * 获取基础数据
     *
     * @return
     */
    @Override
    public Map<String, Object> getBaseData() {
        Map<String, Object> map = new HashMap<>(1);
        map.put("river_grade_list", CommonConstant.LSADMIN_RIVER_GRADE_LIST);
        return map;
    }
}
