package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.constant.CommonConstant;
import com.taauav.common.bean.Response;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import com.taauav.common.util.StringUtils;
import com.taauav.front.entity.LsAuthGroup;
import com.taauav.front.mapper.LsAuthGroupMapper;
import com.taauav.front.query.LsAuthGroupQuery;
import com.taauav.front.service.ILsAuthGroupService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.taauav.front.vo.LsAuthGroupListVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 外部用户角色表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@Service
public class LsAuthGroupServiceImpl extends ServiceImpl<LsAuthGroupMapper, LsAuthGroup> implements ILsAuthGroupService {

    @Autowired
    private LsAuthGroupMapper lsAuthGroupMapper;

    @Autowired
    private Response response;

    /**
     * 获取数据列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(LsAuthGroupQuery query) {
        // 查询条件
        QueryWrapper<LsAuthGroup> queryWrapper = new QueryWrapper<>();
        // 角色名称
        if (!StringUtils.isEmpty(query.getName())) {
            queryWrapper.like("name", query.getName());
        }
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        // 查询数据
        IPage<LsAuthGroup> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<LsAuthGroup> data = lsAuthGroupMapper.selectPage(page, queryWrapper);
        List<LsAuthGroup> lsAuthGroupList = data.getRecords();
        List<LsAuthGroupListVo> lsAuthGroupListVoList = new ArrayList<>();
        if (!lsAuthGroupList.isEmpty()) {
            lsAuthGroupList.forEach(item -> {
                LsAuthGroupListVo lsAuthGroupListVo = new LsAuthGroupListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, lsAuthGroupListVo);
                lsAuthGroupListVoList.add(lsAuthGroupListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", lsAuthGroupListVoList);
        return response.success(result);
    }

    /**
     * 添加角色
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(LsAuthGroup entity) {
        entity.setCreateUser(ShiroUtils.getAdminId());
        entity.setCreateTime(FunctionUtils.getCurrentTime());
        int result = lsAuthGroupMapper.insert(entity);
        if (result == 0) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 获取详情
     *
     * @param id 角色ID
     * @return
     */
    @Override
    public Response info(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("角色ID不存在");
        }
        LsAuthGroup entity = lsAuthGroupMapper.selectById(id);
        return response.success(entity);
    }

    /**
     * 编辑角色
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(LsAuthGroup entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("角色ID不存在");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(FunctionUtils.getCurrentTime());
        int result = lsAuthGroupMapper.updateById(entity);
        if (result == 0) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 删除角色
     *
     * @param id 角色ID
     * @return
     */
    @Override
    public Response delete(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("角色ID不存在");
        }
        LsAuthGroup entity = lsAuthGroupMapper.selectById(id);
        if (entity == null) {
            return response.failure("角色信息不存在");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(FunctionUtils.getCurrentTime());
        entity.setMark(0);
        int result = lsAuthGroupMapper.updateById(entity);
        if (result == 0) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response setStatus(LsAuthGroup entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("角色ID不存在");
        }
        if (entity.getStatus() == null || entity.getStatus() <= 0) {
            return response.failure("角色状态不能为空");
        }
        int result = lsAuthGroupMapper.updateById(entity);
        if (result == 0) {
            return response.failure("设置失败");
        }
        return response.success("设置成功");
    }

    /**
     * 获取角色列表
     *
     * @return
     */
    @Override
    public Response getAuthGroupList() {
        // 查询条件
        QueryWrapper<LsAuthGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", 1);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        // 查询数据
        List<LsAuthGroup> lsAuthGroupList = lsAuthGroupMapper.selectList(queryWrapper);
        return response.success(lsAuthGroupList);
    }

}
